
/*
  trash-drawer moved by stepper motor with S-curve velocity profile.
  Motor: 200steps/turn
  Motordriver: 16 mictrosteps per step
  20teeth pulley, T5 belt
  
  Arduino pro-mini with ATmega328 at 16MHz
 */
	
#include <avr/pgmspace.h>	
#include "delaytable.h"

#define AI_Versorgung 	0  //Pin A0  
#define AI_Nullposition 1  //Pin A1
#define DO_DISABLE 	2
#define DO_DIR 		3
#define DO_STEP 	4
#define DO_SensorPull 	8
#define DI_SENSOR 	9
#define DO_RELAIS 	10
#define DO_LED 		13

#define BIT0 1
#define BIT1 2
#define BIT2 4
#define BIT3 8
#define BIT4 16
#define BIT5 32
#define BIT6 64
#define BIT7 128

#define DISABLE_L     PORTD 	&= ~BIT2
#define DISABLE_H     PORTD 	|=  BIT2
#define DIR_L         PORTD	&= ~BIT3
#define DIR_H         PORTD	|=  BIT3
#define STEP_L        PORTD     &= ~BIT4
#define STEP_H        PORTD 	|=  BIT4
#define SENSOR_PULL_L PORTB     &= ~BIT0      
#define SENSOR_PULL_H PORTB     |=  BIT0      
#define RELAIS_L      PORTB 	&= ~BIT2
#define RELAIS_H      PORTB 	|=  BIT2
#define LED_L         PORTB 	&= ~BIT5
#define LED_H         PORTB 	|=  BIT5
#define SENSOR_IN     (PINB & BIT1)

//---global---
unsigned int volatile index;
unsigned int tableLen;
byte volatile stage;

 //--------------------------------------------------------------------------------------
 void delay_ms(unsigned int ms){
 //--------------------------------------------------------------------------------------
 unsigned int i;
   while (ms--){
     for (i=0; i<3200; i++){
        asm volatile ("nop"); 
     }
   }
 }



//--------------------------------------------------------------------------------------
ISR(TIMER1_COMPA_vect){//timer1 interrupt
//--------------------------------------------------------------------------------------
  if (stage){
      STEP_H;
      if (1 == stage){
        OCR1A  = pgm_read_word_near(delayTable + index); //set delay to next step
        index++;
        if (index >= tableLen){
          stage = 2;  
        }
      }
      if (2 == stage){
        OCR1A  = pgm_read_word_near(delayTable + index); //set delay to next step
        index--;
        if (0==index){
          OCR1A  = 20000;
          stage = 0;  
        }
      }
      STEP_L;   //Pulse is about 2µs long
  }
}
//--------------------------------------------------------------------------------------
void moveDrawer(void){
//--------------------------------------------------------------------------------------

  RELAIS_H; //open motor short circuit 
  delay_ms(15);
  DISABLE_L; //driver on
  delay_ms(10);
  stage = 1;
  index = 0;

  //--- drive --- 
  sei();//allow interrupts
  while(stage){ //wait until move complete
    sensor();
  }
  cli();//disable interrupts
  delay_ms(100); //give drawer time to stop swinging
  DISABLE_H; //driver off
  delay_ms(50);
  RELAIS_L;   //short circuit the motor to avoid reverse voltage (killing the driver chip) if drawer is moved manually
  delay_ms(50);
}

//--------------------------------------------------------------------------------------
bool sensor(void){
//--------------------------------------------------------------------------------------
  //Sensor: calculate capacity of capacitor consisting of wire and earth
  //1. Pullup-Output pin to LOW. This pin is connected with 560K resistor to Sensor-pin.
  //2. Sensor-Pin as output with High-level. Wait some short time.
  //3. Switch sensor pin to input
  //4. count cycles until input changes from high to low (cnt)
  //5. Pullup-Output pin to High.
  //6. count cycles until input changes from low to high
  //   If you touch the wire at the sensor pin, the capacity against earth increases and so
  //   the cnt reaches a higher value.
  byte cnt=0;
 
  pinMode(DO_SensorPull, OUTPUT);
  pinMode(DI_SENSOR, OUTPUT);
  digitalWrite(DO_SensorPull, LOW);  
  digitalWrite(DI_SENSOR, HIGH);  
  delay_ms(3); // Let cable get the High level (charge)
  pinMode(DI_SENSOR, INPUT);  //Pin connected via 560K to PIN8, which is LOW. (Discharge)
  while(digitalRead(DI_SENSOR)){
    cnt++;
  }
  delay_ms(3); // Let cable get the LOW level (charge)
  digitalWrite(DO_SensorPull, HIGH);  
  while(!digitalRead(DI_SENSOR)){
    cnt++;
  }
  pinMode(DI_SENSOR, OUTPUT);
  digitalWrite(DI_SENSOR, LOW);  
  digitalWrite(DO_SensorPull, LOW);  
  
  if (cnt<7){ 
    digitalWrite(DO_LED, LOW);  
    return false; 
  }else{
    digitalWrite(DO_LED, HIGH);   
    return true;
  }
}

//--------------------------------------------------------------------------------------
void setup() {  // the setup function runs once when you press reset or power the board
//--------------------------------------------------------------------------------------
 cli();//stop interrupts
  DDRB = BIT0|BIT2|BIT5;
  DDRD = BIT2|BIT3|BIT4|BIT7;
  PORTD |= BIT2; //disable Stepper
  STEP_L;

  TCCR1A = 0;
  TCCR1B = (1<<WGM12)|(1<<CS11); //CTC-Mode  /8 = 2MHz Takt
  TCCR1C = 0;
  OCR1A=60000;
  TIMSK1=(1<<OCIE1A);
  TIMSK0=0;
  TIMSK2=0;
  PCICR=0;
  LED_H;
  delay_ms(100);
  LED_L;
  TIFR1=(1<<OCF1A); //Clear OCF1A
  tableLen = sizeof(delayTable)/2-2;
  stage=0;
  index=0;
//  sei();//allow interrupts
  
  //SilentStick kalibrieren
  RELAIS_H; //Stepper nicht mehr kurzschliessen
  delay_ms(25);
  DISABLE_L; //Stepper einschalten
  delay_ms(250);
  DISABLE_H; //Stepper ausschalten
  delay_ms(30);
  RELAIS_L; //Stepper  kurzschliessen
  if (sensor()){ //Touch sensor at startup to close drawer after installing Arduino
      DIR_H; // direction: close
      moveDrawer();
  }
}


//--------------------------------------------------------------------------------------
void loop() {    // the loop function runs over and over again forever
//--------------------------------------------------------------------------------------
static byte s=0;  
static byte mainState=0;

  switch(mainState){
    case 0:  
      if (sensor()) s++; else s=0;
      if (s > 3){  
	 DIR_L; // direction: open
         moveDrawer();
         s=0;
         mainState=1;
      }
      break;

    case 1:     
      if (!sensor()) s++; else s=0;
      if (s>20){
        s=0; 
        mainState=2;
      }
      break;
	  
    case 2:  
      if (sensor()) s++; else s=0;
      if (s > 3){  
         DIR_H; // direction: close
         moveDrawer();
         s=0;
         mainState=3;
      }
      break;
	  
    case 3:     
      if (!sensor()) s++; else s=0;
      if (s>20){
        s=0; 
        mainState=0;
      }
      break;
  }
}

